@@ -2,7 +2,7 @@
 
 #
 # Written by Sébastien Millet
-# July, August, September 2016
+# July - October 2016
 #
 
 #
@@ -12,17 +12,9 @@
 use strict;
 use warnings;
 
-use lib '../lib';
-
-my $PRG = "csvcopy.pl";
-
 use Getopt::Long;
 use Text::AutoCSV;
 
-#use feature 'say';
-#use Data::Dumper;
-#$Data::Dumper::Sortkeys = 1;
-
 my $infoh = \*STDERR;
 
 	#
@@ -70,6 +62,12 @@ my @PARAMS = (
 		"0 to remove feature \"always quote\", 1 to enforce it\n" .
 		"(default: same as input)"],
 
+	['out_fields=s', 'ARRAYREF', 'out_fields', '<same fields as input>',
+		"list of fields to write to output\n" .
+		"you separate field names with ||\n" .
+		"as in \"NAME||ADDRESS\"\n" .
+		"(default: none)"],
+
 	['dont_mess_with_encoding=i', 'BOOL', 'dont_mess_with_encoding', 'no',
 		"if set to 1, completely ignore encoding aspects, meaning,\n" .
 		"leave it to perl default\n" .
@@ -120,9 +118,20 @@ my @PARAMS = (
 		"(default: auto-detect date and time)",
 	1],
 
+	['join|j=s', 'STR', undef, '<no join>',
+		"Join input with another CSV file\n" .
+		"Argument is formed as \"PREFIX::LOCAL->REMOTE::FILE\"\n" .
+		"where PREFIX will be added at beginning at each field of joined file,\n" .
+		"where LOCAL is the main input field to read,\n" .
+		"where REMOTE is the joined file field to find,\n" .
+		"and FILE is the joined file.\n" .
+		"Example: \"1::LOGIN->SHORTNAME::users.csv\"\n" .
+		"Options can be appended with \"::{ ...options... }\" as in\n" .
+		"\"1::LOGIN->SHORTNAME::users.csv::{ignore_empty => 0}\"\n" .
+		"(default: no join)"],
+
 	['croak_if_error=i', 'BOOL', 'croak_if_error', 'yes',
-		"croak if an error occurs in Text::AutoCSV (useless in\n" .
-		"this script, put here for the sake of pedagogy)"],
+		"croak if an error occurs in Text::AutoCSV"],
 
 	['debug|d', 'BOOL', '_debug', 'no', 'debug output'],
 );
@@ -136,14 +145,15 @@ for my $p (@PARAMS) {
 }
 
 Getopt::Long::Configure('no_ignore_case', 'bundling');
-usage() unless GetOptions(%opts);
+veryshortusage() unless GetOptions(%opts);
 
-&usage() if ($opts_vals{'help'});
-&usage(1) if ($opts_vals{'Help'});
+usage() if ($opts_vals{'help'});
+usage(1) if ($opts_vals{'Help'});
 if (@ARGV) {
 	print($infoh "Trailing options\n");
-	&usage();
+	veryshortusage();
 }
+my $verbose = $opts_vals{'verbose'};
 
 my $opt_name_max_length = -1;
 for my $p (@PARAMS) {
@@ -163,6 +173,10 @@ for my $p (@PARAMS) {
 
 $opts_vals{dates_auto} = 0 if !defined($opts_vals{dates_auto}) and defined($opts_vals{dates});
 
+#
+# Print out options value if verbose is set
+#
+
 for my $p (@PARAMS) {
 	next unless defined($p->[2]);
 
@@ -182,10 +196,14 @@ for my $p (@PARAMS) {
 	my $opt_disp = opt_to_disp($p->[0], 1);
 
 	printf($infoh "%-${opt_name_max_length}s %s %s\n", $opt_disp, (defined($v) ? '*' : '.'), $vstr)
-		if $opts_vals{'verbose'} and !$opts_vals{'id'};
+		if $verbose and !$opts_vals{'id'};
 
 }
 
+#
+# Work out Text::AutoCSV attributes
+#
+
 my %opts_autocsv;
 for my $p (@PARAMS) {
 	my $attribute = $p->[2];
@@ -198,18 +216,53 @@ for my $p (@PARAMS) {
 	$opts_autocsv{$attribute} = $v if defined($v);
 }
 
+#
+# Create object
+#
+
 my $csv = Text::AutoCSV->new(%opts_autocsv);
 
+#
+# Manage join
+#
+
+if (defined($opts_vals{'join'})) {
+	my ($prefix, $local, $remote, $file, $opts_read);
+	unless (($prefix, $local, $remote, $file, undef, $opts_read) =
+			$opts_vals{'join'} =~ m/^([^,]*),([^,]+)->([^,]+),([^,]+)(,(\{.*\}))?$/) {
+		print($infoh "Invalid join string\n");
+		veryshortusage();
+	}
+	$opts_read = '' unless defined($opts_read);
+
+	print($infoh "** join with $file:\n   prefix: $prefix\n   local field: $local\n" .
+		"   remote field: $remote\n   options: $opts_read\n") if $verbose;
+
+	my $opts = { };
+	if ($opts_read ne '') {
+		eval "\$opts = $opts_read";
+		if (ref($opts) ne 'HASH') {
+			print($infoh "Invalid join options\n");
+			veryshortusage();
+		}
+	}
+	$csv->join($prefix, "${local}->${remote}", $file, $opts);
+}
+
+#
+# "Execute" object, that means, either get description of columns or trigger write
+#
+
 if (!$opts_vals{'id'}) {
 
 	$csv->write();
 
 } else {
 
-	printf("sep_char:            " . $csv->get_sep_char() . "\n");
-	printf("escape_char:         " . $csv->get_escape_char() . "\n");
-	printf("current_in_encoding: " . $csv->get_current_in_encoding() . "\n");
-	printf("is_always_quoted:    " . ($csv->get_is_always_quoted() ? 'yes' : 'no') . "\n");
+	printf("sep_char:         " . $csv->get_sep_char() . "\n");
+	printf("escape_char:      " . $csv->get_escape_char() . "\n");
+	printf("in_encoding:      " . $csv->get_in_encoding() . "\n");
+	printf("is_always_quoted: " . ($csv->get_is_always_quoted() ? 'yes' : 'no') . "\n");
 
 	my @coldata = $csv->get_coldata();
 	my @disp;
@@ -231,9 +284,18 @@ if (!$opts_vals{'id'}) {
 	printf("$s\n", @{$_}) for (@disp);
 }
 
+#
+# End of main code.
+# Below are subs.
+#
 
-# SUBS ***********************************************************************
+sub veryshortusage {
+	print $infoh <<'EOF';
+Try 'csvcopy.pl --help' for more information.
+EOF
 
+	exit 1;
+}
 
 sub usage {
 	my $bigger = shift;
@@ -287,11 +349,14 @@ EOF
     encoding is) to UTF-8.
 
   csvcopy.pl -i a.csv -o t.csv --out_dates_format "%FT%T"
-	Copy a.csv into t.csv, enforcing yyyy-mm-ddThh:MM:ss format for any column
-	that contains a datetime.
+    Copy a.csv into t.csv, enforcing yyyy-mm-ddThh:MM:ss format for any column
+    that contains a datetime.
 
   csvcopy.pl -i a.csv -o t.csv --out_dates_format "%b %d, %Y, %I:%M:%S %p" --out_dates_locale "en"
-	Copy a.csv into t.csv, enforcing US datetime format.
+    Copy a.csv into t.csv, enforcing US datetime format.
+
+  csvcopy.pl -i f1.csv -o joined.csv --join "1:,A->B,f2.csv,{case=>1}"
+    Join f2.csv to f1.csv by mathcing f1'A field with f2'B field, case sensitive match.
 EOF
 	exit 0;
 }
